#!/usr/bin/env python
#coding: utf-8

# pidgin.py - pidgin listener part of speechdesktop
# Copyright (C) Bohdan R. Rau 2012 <ethanak@polip.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program.  If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.

import htmlentitydefs,re,time,dbus
import i18n

def _uni(t):
	if isinstance(t,str):
		t = t.decode('utf-8')
	elif not isinstance(t,unicode):
		t = unicode(t)
	return t

def _unescape(text):
    def fixup(m):
        text = m.group(0)
        if text[:2] == "&#":
            # character reference
            try:
                if text[:3] == "&#x":
                    return unichr(int(text[3:-1], 16))
                else:
                    return unichr(int(text[2:-1]))
            except ValueError:
                pass
        elif text[1:-1] == "apos":
		return "'"
	else:
            # named entity
            try:
                text = unichr(htmlentitydefs.name2codepoint[text[1:-1]])
            except KeyError:
                pass
        return text # leave as is
    return re.sub("&#?\w+;", fixup, text)

def _speakable(text):
	if not text:
		return u''
	if not isinstance(text,unicode):
		try:
			text=text.decode('utf-8')
		except:
			return u''
	text=_unescape(re.sub("<.*?>"," ",text))
	text=re.sub('[\[{]','(',text);
	text=re.sub('[\]}]',')',text);
	return text


class Pidgin(object):
	def __init__(self,bus,speaker):

		self.__bus=bus
		self.__purple=None
		self.__timestamp=None
		self.__speaker=speaker

		self.__bus.add_signal_receiver(self.message,
	                        dbus_interface="im.pidgin.purple.PurpleInterface",
	                        signal_name="ReceivedImMsg")
		self.__bus.add_signal_receiver(self.message,
	                        dbus_interface="im.pidgin.purple.PurpleInterface",
	                        signal_name="ReceivedChatMsg")
		self.__bus.add_signal_receiver(self.ctimestamp,
	                        dbus_interface="im.pidgin.purple.PurpleInterface",
	                        signal_name="ConversationTimestamp")
		self.__bus.add_signal_receiver(self.onsignon,
	                        dbus_interface="im.pidgin.purple.PurpleInterface",
	                        signal_name="BuddySignedOn")
		self.__bus.add_signal_receiver(self.onsignoff,
	                        dbus_interface="im.pidgin.purple.PurpleInterface",
	                        signal_name="BuddySignedOff")

		
	@staticmethod
	def split_digits(alias):
		cl=[]
		while len(alias) > 4:
			cl.insert(alias[-3:])
			alias=alias[:-3]
		if len(alias) == 4:
			cl.insert(alias[0:2]+' '+alias[2:])
		else:
			cl.insert(alias)
		return ' '.join(cl)

	def start_bus(self,restart=False):
		if not restart and self.__purple:
			return True
		try:
			obj = self.__bus.get_object(
				"im.pidgin.purple.PurpleService", "/im/pidgin/purple/PurpleObject")
			self.__purple = dbus.Interface(obj, "im.pidgin.purple.PurpleInterface")
			return True
		except:
			return False

	def speak(self,text):
		self.__speaker.say(text)
	
	def message(self,account, sender, message, conversation, flags):
		self.__speaker.refresh()
		if not self.__speaker.get_conf_b('pidgin','message',True):
			return
		message=_speakable(message)
		try:
			if not self.start_bus():
				return
			buddy = self.__purple.PurpleFindBuddy(account, sender)
		except:
			#import traceback
			#traceback.print_exc()
			try:
				if not self.start_bus(True):
					return
				buddy = self.__purple.PurpleFindBuddy(account, sender)
			except:
				return
		if buddy != 0:
			try:
				alias = self.__purple.PurpleBuddyGetAlias(buddy)
			except:
				alias = sender
		else:
			alias = sender
			
		alias=_uni(alias)
		if re.match('^[0-9]+$',alias) and len(alias)>3:
			alias=self.split_digits(alias)
		self.speak(i18n.trans(self.__speaker.lang(),'pidgin','message') % (alias,message))
	
	def ctimestamp(self,account,timestamp,ignore):
		self.__timestamp=time.localtime(timestamp)
		
	def get_buddy_alias(self,buddy):
		try:
			if not self.start_bus():
				return None
			alias = self.__purple.PurpleBuddyGetAlias(buddy)
		except:
			try:
				if not self.start_bus(True):
					return None
				alias = self.__purple.PurpleBuddyGetAlias(buddy)
			except:
				return None
		buddy=_uni(buddy)
		if re.match('^[0-9]+$',alias) and len(alias)>3:
			alias=self.split_digits(alias)
		return alias
		
	def onsignon(self,buddy):
		self.__speaker.refresh()
		if not self.__speaker.get_conf_b('pidgin','announce',True):
			return
		alias=self.get_buddy_alias(buddy)
		if alias:
			self.speak(i18n.trans(self.__speaker.lang(),'pidgin','buddyin') % (alias,))
			
	def onsignoff(self,buddy):
		self.__speaker.refresh()
		if not self.__speaker.get_conf_b('pidgin','announce',True):
			return
		alias=self.get_buddy_alias(buddy)
		if alias:
			self.speak(i18n.trans(self.__speaker.lang(),'pidgin','buddyout') % (alias,))


def simplerun():
	"""
	Requires gir1.2-glib-2.0
	"""
	from dbus.mainloop.glib import DBusGMainLoop
	from gi.repository import GLib
	import speech
	
	speaker=speech.SpdsCore(True)
	dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
	bus = dbus.SessionBus()
	pidgin=Pidgin(bus,speaker)
	loop=GLib.MainLoop()
	
	try:
		loop.run()
	except:
		pass
	speaker.close()

if __name__ == '__main__':
	simplerun()
