#!/usr/bin/env python
#coding: utf-8

# speech.py - high level speech interface for speechdesktop
# Copyright (C) Bohdan R. Rau 2012 <ethanak@polip.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program.  If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.

import re, os, os.path
from speechd import SSIPClient,Scope

class SpdsException(Exception):
	pass

class SpdsConf(object):
	_r1=re.compile(r'\[(.*)\]$')
	def __init__(self,filename=None,allownoex=True):
		self.__watchtime=None
		if filename:
			self.read(filename,allownoex)
			self.__filename=filename
		else:
			self.__vars={}
			self.__sections=[]
			self.__filename=None

	def refresh(self,enforce=False):
		if not self.__filename:
			return False
		try:
			s=os.stat(self.__filename)
		except:
			return False
		if enforce or self.__watchtime != s.st_mtime:
			self.__watchtime=s.st_mtime
			self.read()
			return True
		return False
	
	def read(self,filename=None,allownoex=False):
		if not filename:
			filename=self.__filename
			if not filename:
				raise SpdsException('Filename not given')
		else:
			self.__filename=filename
		tvar=None
		self.__vars={}
		self.__sections=[]
		if self.__watchtime is None:
			try:
				s=os.stat(filename)
			except:
				if allownoex:
					return
				raise SpdsException('File does not exist')
			self.__watchtime=s.st_mtime
		try:
			f=open(filename)
		except:
			if allownoex:
				return
			raise SpdsException('File does not exist')
		while True:
			line=f.readline()
			if not line:
				break
			line=line.strip()
			if line.startswith('#') or line == '':
				continue
			if line.startswith('['):
				t=self.__class__._r1.match(line)
				if not t:
					raise SpdsException('Bad section line')
				t=t.group(1).strip()
				if not t:
					raise SpdsException('Bad section line')
				if t in self.__vars:
					raise SpdsException('Repeated section name')
				tvar=t
				self.__sections.append(tvar)
				self.__vars[t]={}
				continue
			line=line.split('=',1)
			if len(line) != 2:
				raise SpdsException('Bad key=value line')
			line[0]=line[0].strip()
			line[1]=line[1].strip()
			if not line[0]:
				raise SpdsException('Empty key')
			if not tvar:
				raise SpdsException('Key without section')
			self.__vars[tvar][line[0]] = line[1]
		f.close()
		
	
	def get(self,section,key,defval=None):
		section=str(section)
		if section not in self.__vars:
			return defval
		return self.__vars[section].get(key,defval)
	
	def set(self,section,key,value):
		section=str(section)
		if section not in self.__vars:
			self.__vars[section]={}
			self.__sections.append(section)
		key=str(key)
		if isinstance(value,unicode):
			value=value.encode('utf-8')
		else:
			value=str(value)
		self.__vars[section][key]=value
				
	def save(self,filename=None):
		if not filename:
			filename=self.__filename
			if not filename:
				raise SpdsException('Filename not given')
		f=open(filename,'w')
		for section in self.__sections:
			f.write('[%s]\n' % (section,))
			for key in self.__vars[section]:
				f.write('%s=%s\n' %(key,self.__vars[section][key]))
		f.close()

class SpdsCore(object):
	
	@staticmethod
	def ensure(val,mini,maxi):
		if (val < mini):
			return mini
		if (val > maxi):
			return maxi
		return val
	
	def __init__(self,watch=False):
		self.__watch=watch
		home=os.environ['HOME']
		fname=os.path.join(home,'.speechdesktop.cfg')
		self.conf=SpdsConf(fname,True)
		self.__client=None
		self.__spd_module=self.conf.get('speechd','module',None)
		self.__spd_voice=self.conf.get('speechd','voice',None)
		self.__spd_rate=self.conf.get('speechd','rate',None)
		self.__lang='en'
		self.connect()
	
	def _init_modules(self):
		modules=self.get_modules()
		module=self.conf.get('speechd','module',None)
		voice=None
		voices=None
		if not (module in modules):
			module=None
		else:
			self.set_module(module)
			voice=self.conf.get('speechd','voice',None)
			voices=self.get_voices()
			found=False
			for a in voices:
				if a[0] == voice:
					found=True
			if not found:
				voice=None
		if voice:
			return
		import i18n
		lang=i18n.curlang()[:2]
		if module:
			# search for voice with current language
			for a in voices:
				if a[1] == lang:
					voice=a[0]
					break
		if not voice:
			# search with all modules
			for mod in modules:
				if mod == module:
					continue
				self.set_module(mod)
				voices=self.get_voices()
				for a in voices:
					if a[1] == lang:
						voice=a[0]
						break
				if voice:
					module=mod
					break
		if not voice and lang != 'en':
			#search for english in all modules
			for mod in modules:
				self.set_module(mod)
				voices=self.get_voices()
				for a in voices:
					if a[1] == 'en':
						voice=a[0]
						break
				if voice:
					module=mod
					break
		if not voice:
			for mod in modules:
				self.set_module(mod)
				voices=self.get_voices()
				if not voices:
					continue
				module=mod
				voice=voices[0][0]
				break
		if not voice:
			raise SpdsException('Cannot find usable module')
		self.conf.set('speechd','voice',voice)
		self.conf.set('speechd','module',module)
		
	def connect(self):
		if self.__client:
			self.__client.close()
		self.__client=SSIPClient('speechdesktop')
		self._init_modules()
		val=self.conf.get('speechd','module',None)
		if val:
			self.set_module(val)
			val=self.conf.get('speechd','voice',None)
			if val:
				self.set_voice(val)
		rate=self.conf.get('speechd','rate',None)
		if rate:
			self.set_rate(rate)

		pitch=self.conf.get('speechd','pitch',None)
		if pitch:
			self.set_pitch(rate)
		vol=self.conf.get('speechd','vol',None)
		if vol:
			self.set_volume(vol)
		
	
	def stop(self):
		try:
			self.__client.cancel(Scope.ALL)
		except:
			pass
		
	def refresh(self,enforce=False):
		if not self.__watch and not enforce:
			return
		if not self.conf.refresh(enforce):
			return
		try:
			if enforce or self.__spd_module != self.conf.get('speechd', 'module', None):
				self.__spd_module = self.conf.get('speechd', 'module', None)
				if self.__spd_module:
					self.set_module(self.__spd_module)
				self.__spd_voice = None
			
			if enforce or self.__spd_voice != self.conf.get('speechd', 'voice', None):
				self.__spd_voice = self.conf.get('speechd', 'voice', None)
				if self.__spd_voice:
					self.set_voice(self.__spd_voice)
			
			if enforce or self.__spd_rate != self.conf.get('speechd', 'rate', None):
				self.__spd_rate = self.conf.get('speechd', 'rate', None)
				if self.__spd_rate is not None:
					self.__client.set_rate(int(self.__spd_rate))

			if enforce or self.__spd_pitch != self.conf.get('speechd', 'pitch', None):
				self.__spd_pitch = self.conf.get('speechd', 'pitch', None)
				if self.__spd_pitch is not None:
					self.__client.set_pitch(int(self.__spd_pitch))
			if enforce or self.__spd_pitch != self.conf.get('speechd', 'volume', None):
				self.__spd_volume = self.conf.get('speechd', 'volume', None)
				if self.__spd_volume is not None:
					self.__client.set_volume(int(self.__spd_volume))
		except:
			pass
			
	def say(self,text,breaking=False):
		if breaking:
			self.stop()
		self.refresh()
		try:
			self.__client.speak(text)
		except:
			self.connect()
			try:
				self.__client.speak(text)
			except:
				pass
		
	def close(self):
		if self.__client:
			self.__client.close()
		self.__client=None
		
	def get_conf(self,section,key,defval=None):
		return self.conf.get(section,key,defval)

	def set_conf(self,section,key,value):
		self.conf.set(section,key,value)
	
	def get_conf_i(self,section,key,defval=0):
		try:
			val=self.conf.get(section,key,str(defval))
		except:
			return defval
		return int(val)
		
	def get_conf_b(self,section,key,defval=False):
		if not defval:
			defval='n'
		else:
			defval='y'
		try:
			val=self.conf.get(section,key,defval).lower()
		except:
			return False
		return val in ('yes','on','true','1','y','t')
	
	def save_conf(self):
		self.conf.save()
		
	def get_modules(self):
		try:
			ls=self.__client.list_output_modules()
		except:
			try:
				self.__client.connect()
				ls=self.__client.list_output_modules()				
			except:
				return None
		lt=[]
		for a in ls:
			if a != 'dummy':
				lt.append(a)
		lt.sort()
		return lt

	def set_module(self,module):
		self.__client.set_output_module(module)
		
	def ltsort(self,a,b):
		a=a[0]
		b=b[0]
		if a < b:
			return -1
		if a > b:
			return 1
		return 0
	
	def set_voice(self,voice,lang=None):
		self.__client.set_synthesis_voice(voice)
		if not lang or lang == 'none':
			v=self.get_voices()
			if v:
				for a in v:
					if a[0] == voice:
						lang=a[1]
						break
		if not lang or lang == 'none':
			lang='en'
		self.__lang=lang
		
	def get_voices(self):
		try:
			ls=self.__client.list_synthesis_voices()
		except:
			import traceback
			traceback.print_exc()
			try:
				self.__client.connect()
				ls=self.__client.list_synthesis_voices()
			except:
				import traceback
				traceback.print_exc()
				return None
		lt=[]
		for a in ls:
			lt.append((a[0],a[1]))
		lt.sort(self.ltsort)
		return lt
	
	def set_rate(self,rate,cnf=False):
		if cnf:
			self.set_conf('speechd','rate',rate)
		rate=int(rate)
		rate=self.ensure(rate,-100,100)
		self.__client.set_rate(rate)

	def set_pitch(self,pitch,cnf=False):
		if cnf:
			self.set_conf('speechd','pitch',pitch)
		pitch=int(pitch)
		pitch=self.ensure(pitch,-100,100)
		self.__client.set_pitch(pitch)

	def set_volume(self,vol,cnf=False):
		if cnf:
			self.set_conf('speechd','volume',vol)
		vol=int(vol)
		vol=self.ensure(vol,-100,100)
		self.__client.set_volume(vol)
		
	
	def lang(self):
		return self.__lang
		
def fastStop():
	client=SSIPClient('speechdesktop')
	client.cancel(Scope.ALL)
	client.close()
	
